--  pg2-interface.adb
--  Mario Amado Alves
--  2000-01-18

with Interfaces.C; use Interfaces.C;
with Interfaces.C.Strings; use Interfaces.C.Strings;

package body Pg2.Inter_face is


--  MAIN FUNCTIONS

   procedure Disconnect is
   begin
      C_Disconnect;
   end Disconnect;

   function Submit (
     Command : in Unbounded_String;
     Result : in Index_Type)
     return Index_Type is
   begin
      return (To_Ada_Index (C_Submit (
        To_C_String (Command),
        To_C_Int (Result))));
   end Submit;

   function Get (
     Result : in Index_Type;
     Tuple : in Index_Type;
     Field : in Index_Type)
     return Unbounded_String is
   begin
      return (To_Ada_Unbounded_String (C_Get (
       To_C_Int (Result),
       To_C_Int (Tuple),
       To_C_Int (Field))));
   end Get;


   --  RESULT OF LAST OPERATION

   function Last_OK return Boolean is
   begin
      return (To_Boolean (C_Last_OK));
   end Last_OK;

   function Error_Message return Unbounded_String is
   begin
      return (To_Ada_Unbounded_String (C_Error_Message));
   end Error_Message;


   --  CONNECTION INFO

   function Host_Info return Unbounded_String is
   begin
      return (To_Ada_Unbounded_String (C_Host_Info));
   end Host_Info;

   function Port_Info return Unbounded_String is
   begin
      return (To_Ada_Unbounded_String (C_Port_Info));
   end Port_Info;

   function Options_Info return Unbounded_String is
   begin
      return (To_Ada_Unbounded_String (C_Options_Info));
   end Options_Info;

   function TTY_Info return Unbounded_String is
   begin
      return (To_Ada_Unbounded_String (C_TTY_Info));
   end TTY_Info;

   function Database_Info return Unbounded_String is
   begin
      return (To_Ada_Unbounded_String (C_Database_Info));
   end Database_Info;


   --  COMMAND RESULT INFO

   function OK (Result : Index_Type) return Boolean is
   begin
      return (To_Boolean (C_OK (To_C_Int (Result))));
   end OK;

   function Tuples (Result : Index_Type) return Boolean is
   begin
      return (To_Boolean (C_Tuples (To_C_Int (Result))));
   end Tuples;

   function Tuples_Qt (Result : Index_Type) return Qt_Type is
   begin
      return (Qt_Type (C_Tuples_Qt (To_C_Int (Result))));
   end Tuples_Qt;

   function Fields_Qt (Result : Index_Type) return Qt_Type is
   begin
      return (Qt_Type (C_Fields_Qt (To_C_Int (Result))));
   end Fields_Qt;


   --  TYPE CONVERSION FUNCTIONS

   function To_Boolean (Item : int) return Boolean is
   begin
      if (Item = 0) then
         return (False);
      else
         return (True);
      end if;
   end To_Boolean;

   function To_C_Int (Item : Index_Type) return int is
   begin
      return int (Item);
   end To_C_Int;

   function To_Ada_Index (Item : int) return Index_Type is
   begin
      return (Index_Type (Item));
   end To_Ada_Index;

   function To_C_String (Item : String) return chars_ptr is
   begin
      return (New_String (Item));
      --  freed where? this is probably leaking memory
   end To_C_String;

   function To_C_String (Item : Unbounded_String) return chars_ptr is
   begin
      return (To_C_String (To_String (Item)));
   end To_C_String;

   function To_Ada_String (Item : chars_ptr) return String is
   begin
      return (Value (Item));
   end To_Ada_String;

   function To_Ada_Unbounded_String (Item : chars_ptr) return Unbounded_String
   is
   begin
      return (To_Unbounded_String (To_Ada_String (Item)));
   exception
      when Interfaces.C.Strings.Dereference_Error =>
         return (Null_Unbounded_String);
   end To_Ada_Unbounded_String;

end Pg2.Inter_face;


