--  pg2-interface.ads
--  Mario Amado Alves
--  2000-01-18

with Interfaces.C; use Interfaces.C;
with Interfaces.C.Strings; use Interfaces.C.Strings;

generic package Pg2.Inter_face is

   pragma Linker_Options ("pg2_interface.o");
   --  pragma Linker_Options ("/usr/lib/libpq.a");
   --  pragma Linker_Options ("/usr/lib/x86_64-linux-gnu/libcrypt.a");

   procedure C_Connect (Parameters : chars_ptr);
   pragma Import (C, C_Connect, "Connect");

   procedure Disconnect;
   procedure C_Disconnect;
   pragma Import (C, C_Disconnect, "Disconnect");

   function Submit (
     Command : in Unbounded_String;
     Result : in Index_Type)
     return Index_Type;
   function C_Submit (
     Command : chars_ptr;
     Result : int)
     return int;
   pragma Import (C, C_Submit, "Submit");

   function Get (
     Result : in Index_Type;
     Tuple : in Index_Type;
     Field : in Index_Type)
     return Unbounded_String;
   function C_Get (
     Result : in int;
     Tuple : in int;
     Field : in int)
     return chars_ptr;
   pragma Import (C, C_Get, "Get");

   procedure C_Forget (Result : in int);
   pragma Import (C, C_Forget, "Forget");

   --  RESULT OF LAST OPERATION

   function Last_OK return Boolean;
   function C_Last_OK return int;
   pragma Import (C, C_Last_OK, "Last_OK");

   function Error_Message return Unbounded_String;
   function C_Error_Message return chars_ptr;
   pragma Import (C, C_Error_Message, "Error_Message");


   --  CONNECTION INFO

   function Host_Info return Unbounded_String;
   function Port_Info return Unbounded_String;
   function Options_Info return Unbounded_String;
   function TTY_Info return Unbounded_String;
   function Database_Info return Unbounded_String;

   function C_Host_Info return chars_ptr;
   function C_Port_Info return chars_ptr;
   function C_Options_Info return chars_ptr;
   function C_TTY_Info return chars_ptr;
   function C_Database_Info return chars_ptr;

   pragma Import (C, C_Host_Info, "Host_Info");
   pragma Import (C, C_Port_Info, "Port_Info");
   pragma Import (C, C_Options_Info, "Options_Info");
   pragma Import (C, C_TTY_Info, "TTY_Info");
   pragma Import (C, C_Database_Info, "Database_Info");


   --  COMMAND RESULT INFO

   function OK (Result : Index_Type) return Boolean;
   function C_OK (Result : int) return int;
   pragma Import (C, C_OK, "OK");

   function Tuples (Result : Index_Type) return Boolean;
   function C_Tuples (Result : int) return int;
   pragma Import (C, C_Tuples, "Tuples");

   function Tuples_Qt (Result : Index_Type) return Qt_Type;
   function C_Tuples_Qt (Result : int) return int;
   pragma Import (C, C_Tuples_Qt, "Tuples_Qt");

   function Fields_Qt (Result : Index_Type) return Qt_Type;
   function C_Fields_Qt (Result : int) return int;
   pragma Import (C, C_Fields_Qt, "Fields_Qt");


   --  TYPE CONVERSION FUNCTIONS

   function To_Boolean (Item : int) return Boolean;
   function To_C_Int (Item : Index_Type) return int;
   function To_Ada_Index (Item : int) return Index_Type;
   function To_C_String (Item : String) return chars_ptr;
   function To_C_String (Item : Unbounded_String) return chars_ptr;
   function To_Ada_String (Item : chars_ptr) return String;
   function To_Ada_Unbounded_String (Item : chars_ptr) return Unbounded_String;

end Pg2.Inter_face;
