--  pg2.ads
--  Mario Amado Alves
--  2000-01-18

with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;


generic

   Parameters : String;
   --  Set of name='value' pairs, separated by space:
   --    host     = host to connect to
   --    port     = port number to connect to at the server host
   --    dbname   = database name
   --    user     = user name for authentication
   --    password = password used if the backend demands authentication
   --    options  = trace/debug options to send to backend
   --    tty      = file or tty for optional debug output from backend

package Pg2 is

   subtype Index_Type is Natural;

   subtype Qt_Type is Natural;

   type Result_Type is record
      OK : Boolean := False;
      Message : Unbounded_String;
      Tuples : Boolean := False;
      Tuples_Qt : Qt_Type := 0;
      Fields_Qt : Qt_Type := 0;
      Index : Index_Type range 0 .. 500 := 500;
   end record;

   procedure Disconnect;

   procedure Submit (
     Command : in Unbounded_String;
     Result : in out Result_Type);

   procedure Get (
     Result : in out Result_Type;
     Tuple : in Index_Type;
     Field : in Index_Type;
     Data : out Unbounded_String);

   procedure Forget (Result : in out Result_Type);

   procedure Escape (S : in out Unbounded_String);
   function Escape (S : Unbounded_String) return Unbounded_String;

   Cannot_Connect : exception;

end Pg2;

--  NOTES
--
--  The string Parameters can be empty to use all default parameters,
--  or it can contain one or more parameter settings separated by
--  whitespace.  Each parameter setting is in the form "keyword=value".
--  To write a null value or a value containing spaces, surround it
--  with single quotes, E.G., "keyword='a value'". Single quotes within
--  the value must be written as "\'". Spaces around the equal sign are
--  optional.
--
--  Currently recognized parameter keywords are:
--
--  "host" : host to connect to. If a non-zero-length string is
--  specified, TCP/IP communication is used. Without a host name, libpq
--  will connect using a local Unix domain socket.
--
--  "port" : port number to connect to at the server host, or socket
--  filename extension for Unix-domain connections.
--
--  "dbname" : database name.
--
--  "user" : user name for authentication.
--
--  "password" : password used if the backend demands password
--  authentication.
--
--  "authtype" : authorization type. (No longer used, since the backend
--  now chooses how to authenticate users. libpq still accepts and
--  ignores this keyword for backward compatibility.)
--
--  "options" : trace/debug options to send to backend.
--
--  "tty" : file or tty for optional debug output from backend.
--
--  Environment variables or built-in default values are use for
--  unspecified options.
--
--  (Adapted from the PostgreSQL Manual.)
